/* Developed for Lenovo Educational Board Pass 3
   © Lenovo Created by Joshua Grosser
   v. 2.0 */

#include <ArduinoBLE.h> // Bluetooth library
#include <Wire.h> // I2C library
#include "ens210.h" // ENS210 library
#include <SparkFun_APDS9960.h> // ambient light library
#include <SSD1306Wire.h> // OLED display library

BLEService ledService("19B10000-E8F2-537E-4F6C-D104768A1214"); // Bluetooth LED Information
BLEByteCharacteristic switchCharacteristic("19B10001-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite); // Bluetooth Readable and Writable UUID
SSD1306Wire display(0x3c);
SparkFun_APDS9960 apds = SparkFun_APDS9960();
ENS210 ens210;
#define WIRE Wire
uint16_t ambient_light = 0;
const int micPin = 10; // microphone pin number
int micVal = 0; // value read from the microphone
uint16_t red_light = 0;
uint16_t green_light = 0;
uint16_t blue_light = 0;

void setup() {
  // Begin serial at 115200 baud rate
  Serial.begin(115200);

  // Initialize Bluetooth
  BLE.begin();

  // Set discoverable name and UUID
  BLE.setLocalName("Lenovo Educational Board");

  // Initialize sensors
  Wire.begin();
  ens210.begin();
  apds.init();
  apds.enableLightSensor(false);
  display.init();
  display.setContrast(255);

  // Setup Bluetooth connections correctly and write value of 0
  BLE.setAdvertisedService(ledService);
  ledService.addCharacteristic(switchCharacteristic);
  BLE.addService(ledService);
  switchCharacteristic.writeValue(0);
  BLE.advertise();
  Serial.println("Bluetooth correctly initialized.");
  display.setLogBuffer(5, 30);
  display.clear();
  display.println("Selections:");
  display.println("1 - Temperature");
  display.println("2 - Ambient Light");
  display.println("3 - Microphone");
  display.println("4 - Colors");
  display.drawLogBuffer(0, 0);
  display.display();
  delay(2);
}

void loop() {
  // Listen for connecting Bluetooth devices
  BLEDevice central = BLE.central();

  // If devices are connected
  if (central) {
    // Print connected device MAC address
    Serial.print("Connected to device: ");
    Serial.println(central.address());

    // While device is connected
    while (central.connected()) {
      // If value is written to Bluetooth
      if (switchCharacteristic.value() == 0) { // A value of 0
        display.setLogBuffer(5, 30);
        display.clear();
        display.println("Selections:");
        display.println("1 - Temperature");
        display.println("2 - Ambient Light");
        display.println("3 - Microphone");
        display.println("4 - Colors");
        display.drawLogBuffer(0, 0);
        display.display();
        delay(2);
      } else if (switchCharacteristic.value() == 1) { // A value of 1
        int t_data, t_status, h_data, h_status;
        ens210.measure(&t_data, &t_status, &h_data, &h_status); 
        // prints output to OLED display
        display.setLogBuffer(5, 30);
        display.clear();
        display.print("T: ");
        display.print(ens210.toFahrenheit(t_data, 10) / 10.0);
        display.print("F , H: ");
        display.print(ens210.toPercentageH(h_data,1));
        display.println("%");
        display.drawLogBuffer(0, 0);
        display.display();
        delay(2);
      } else if (switchCharacteristic.value() == 2) { // A value of 2
        apds.readAmbientLight(ambient_light);
        // prints output to OLED display
        display.setLogBuffer(5, 30);
        display.clear();
        display.print("Ambient Light: ");
        display.println(ambient_light);
        display.drawLogBuffer(0, 0);
        display.display();
        delay(2);
      } else if (switchCharacteristic.value() == 3) { // A value of 3
        micVal = analogRead(micPin);
        // prints output to OLED display
        display.setLogBuffer(5, 30);
        display.clear();
        display.print("Microphone value: ");
        display.println(micVal);
        display.drawLogBuffer(0, 0);
        display.display();
        delay(2);
      } else if (switchCharacteristic.value() == 4) { // A value of 4
      if ( apds.readRedLight(red_light) &&
          apds.readGreenLight(green_light) &&
          apds.readBlueLight(blue_light) ) {
          // print red, green, blue values to display
          display.setLogBuffer(5, 30);
          display.clear();
          display.print(" Red: ");
          display.println(red_light);
          display.print(" Green: ");
          display.println(green_light);
          display.print(" Blue: ");
          display.println(blue_light);
          display.drawLogBuffer(0, 0);
          display.display();
          delay(2);
        }
      }
    }

    // If device disconnects print MAC address
    Serial.print("Disconnected from device: ");
    Serial.println(central.address());
  }
}
